Package: CSSQ
Title: Chip-seq Signal Quantifier Pipeline
Version: 1.24.0
Author: Ashwath Kumar [aut], Michael Y Hu [aut], Yajun Mei [aut], Yuhong Fan [aut]
Maintainer: Fan Lab at Georgia Institute of Technology <yuhong.fan@biology.gatech.edu>
Description: This package is desgined to perform statistical analysis to
    identify statistically significant differentially bound regions between
    multiple groups of ChIP-seq dataset. 
License: Artistic-2.0
Encoding: UTF-8
biocViews: ChIPSeq, DifferentialPeakCalling, Sequencing, Normalization
Depends: SummarizedExperiment, GenomicRanges, IRanges, S4Vectors,
        rtracklayer
Imports: GenomicAlignments, GenomicFeatures, Rsamtools, ggplot2,
        grDevices, stats, utils
Suggests: BiocStyle, knitr, rmarkdown, markdown
VignetteBuilder: knitr
RoxygenNote: 7.0.0
git_url: https://git.bioconductor.org/packages/CSSQ
git_branch: RELEASE_3_23
git_last_commit: e7dea6b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:30:01 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(DBAnalyze)
export(ansTransform)
export(getBgSubVal)
export(getRegionCounts)
export(kmeansNormalize)
export(loadCountData)
export(normalizeData)
export(plotDist)
export(preprocessData)
import(GenomicAlignments)
import(GenomicFeatures)
import(GenomicRanges)
import(IRanges)
import(Rsamtools)
import(SummarizedExperiment)
import(ggplot2)
import(rtracklayer)
importFrom(S4Vectors,DataFrame)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(stats,kmeans)
importFrom(stats,p.adjust)
importFrom(utils,combn)
importFrom(utils,read.table)

#' Perform quantification and normalization of count data
#'
#' This function quantifies each each region for a sample and performs
#' background correction and normalization as instructed. 
#' Returns a vector of count information for the input regions.
#' 
#' @param analysisInfo A 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
#' object with regions and sample information from within 
#' \code{\link{getRegionCounts}}.
#' @param sampleIndex Index of the sample to process.
#' @param normalizeReadDepth Logical indicating if count data should be 
#' normalized for library sequencing depth. When TRUE (default), counts will 
#' be normalized for sequencing depth for each library. When FALSE, no such 
#' normalization will be performed and raw counts will be used. (default: TRUE)
#' @param normalizeLength Logical indicating if count data should be 
#' normalized to the length of the regions. When TRUE, count data will be 
#' normalized for the length of the region being analyzed. 
#' When FALSE (default), no such normalization will be performed. (default: FALSE)
#' @param backgroundSubtract Logical indicating if background correction 
#' should be performed.
#' When TRUE (default), background subtraction will be performed after 
#' length and depth normalization if applicable. When FALSE, no background 
#' subtraction will be performed. (default: TRUE)  
#' @param countMode Count method passed on to 
#' summarizeOverlaps from GenomicAlignments package.
#' (\code{"Union"}, \code{"IntersectionNotEmpty"}, \code{"IntersectionStrict"} 
#' or \code{"user supplied function"}). (default: "Union")
#' @param ignore.strand A logical indicating if strand should be considered 
#' when matching. (default: TRUE)
#' Passed on to summarizeOverlaps from GenomicAlignments package.
#' @param inter.feature A logical indicating if the `r countMode` should be 
#' aware of overlapping features. When TRUE, reads mapping to multiple 
#' features are dropped (i.e., not counted). When FALSE (default), these reads
#' are retained and a count is assigned to each feature they map to. Passed on 
#' to summarizeOverlaps from GenomicAlignments package. (default: FALSE)
#' 
#' @return A vector containing the counts for all the regions. 
#' @seealso \code{\link{getRegionCounts}} which calls this function 
#' @import GenomicRanges
#' @import GenomicFeatures
#' @import SummarizedExperiment
#' @import Rsamtools
#' @export
#' @examples
#' regionBed <- read.table(system.file("extdata", "chr19_regions.bed", 
#' package="CSSQ",mustWork = TRUE))
#' sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
#' package="CSSQ",mustWork = TRUE),sep="\t",header=TRUE)
#' sampleInfo[,3] <- sapply(sampleInfo[,3], 
#' function(x) system.file("extdata", x, package="CSSQ"))
#' sampleInfo[,5] <- sapply(sampleInfo[,5], 
#' function(x) system.file("extdata", x, package="CSSQ"))
#' regionRange <- GRanges(seqnames=regionBed$V1,
#' ranges=IRanges(start=regionBed$V2,end=regionBed$V3))
#' analysisInfo <- SummarizedExperiment(rowRanges=regionRange,
#' colData=sampleInfo)
#' NormbgSubCounts <- data.frame(sapply(c(1:nrow(colData(analysisInfo))),
#' function(x) getBgSubVal(analysisInfo,sampleIndex = x,backgroundSubtract=TRUE,
#' normalizeReadDepth=TRUE,normalizeLength=FALSE,countMode="Union", 
#' ignore.strand=TRUE,inter.feature=FALSE)))
#' NormbgSubCounts

getBgSubVal <- function(analysisInfo,sampleIndex,normalizeReadDepth=TRUE,normalizeLength=FALSE,backgroundSubtract=TRUE,countMode="Union", ignore.strand=TRUE,inter.feature=FALSE){
    params <- ScanBamParam(what = scanBamWhat(), flag = scanBamFlag(isUnmappedQuery = FALSE, isSecondaryAlignment = FALSE, isNotPassingQualityControls = FALSE, isDuplicate = FALSE))
    if (backgroundSubtract == TRUE){
        in_val <- summarizeOverlaps(features=rowRanges(analysisInfo),reads=as.character(colData(analysisInfo)[,5][sampleIndex]),mode=countMode,ignore.strand=TRUE,param=params,inter.feature=inter.feature)
        if (normalizeReadDepth == TRUE){
            in_val <- assay(in_val)*(1000000/colData(analysisInfo)[,6][sampleIndex])
        }
        else{
            in_val <- assay(in_val)[1]
        }
    }
    else{
        in_val <- 0
    }
    ip_val <- summarizeOverlaps(features=rowRanges(analysisInfo),reads=as.character(colData(analysisInfo)[,3][sampleIndex]),mode=countMode,ignore.strand=ignore.strand,param=params,inter.feature=inter.feature)
    if (normalizeReadDepth == TRUE){
        ip_val <- assay(ip_val)*(1000000/colData(analysisInfo)[,4][sampleIndex])
    }
    else{
        ip_val <- assay(ip_val)[1]
    }
    returnVal <- as.numeric(ip_val-in_val)
    if (normalizeLength == TRUE){
        width = width(ranges(rowRanges(analysisInfo)))
        returnVal <- returnVal / width
    }
    return(returnVal)
}

#' Load count data from input file. 
#'
#' It converts input count file and a bed file  regions 
#' into a  \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' object. 
#' 
#' @param countFile A path to file containing the count data for the dataset. 
#' This should be a tab separated file  sample names as header. 
#' @param regionBed A bed file containing the list of regions that are being 
#' analyzed.
#' @param sampleInfo Object from \code{\link{preprocessData}} containing sample
#' information.
#'
#' @return \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' object containing
#' the region information,
#' sample information and the count data. 
#' 
#' @import GenomicRanges
#' @import SummarizedExperiment
#' @importFrom utils read.table
#' @import IRanges
#' @import rtracklayer
#' @export
#' @examples
#' countData <- loadCountData(system.file("extdata", "sample_count_data.txt", 
#' package="CSSQ",mustWork = TRUE),system.file("extdata", "chr19_regions.bed",
#' package="CSSQ"),
#' read.table(system.file("extdata", "sample_info.txt", package="CSSQ",
#' mustWork = TRUE),
#' sep="\t",header=TRUE))
#' countData

loadCountData <- function(countFile,regionBed,sampleInfo) {
    regionRange <- import(regionBed,format="BED")
    countData <- read.table(countFile,sep="\t",header=TRUE)
    analysisInfo <- SummarizedExperiment(assays = list(countData=countData),rowRanges=regionRange,colData=sampleInfo)
    return(analysisInfo)
}

## ----setup_knitr, include = FALSE, cache = FALSE------------------------------
library(BiocStyle)
# Decide whether to display parts for BioC (TRUE) or F1000 (FALSE)
on.bioc <- TRUE
library(knitr)
# Use fig.width = 7 for html and fig.width = 6 for pdf
fig.width <- ifelse(on.bioc, 8, 6)
knitr::opts_chunk$set(cache = 2, warning = FALSE, message = FALSE, error = FALSE,
  cache.path = "cache/", fig.path = "figure/", fig.width = fig.width)

## -----------------------------------------------------------------------------
library(CSSQ)
regionBed <- read.table(system.file("extdata", "chr19_regions.bed", package="CSSQ",mustWork = TRUE))
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", package="CSSQ",mustWork = TRUE),sep="\t",header=TRUE)
head(regionBed)
sampleInfo

## -----------------------------------------------------------------------------
countData <- getRegionCounts(system.file("extdata", "chr19_regions.bed", package="CSSQ"),sampleInfo,sampleDir = system.file("extdata", package="CSSQ"))
countData
head(assays(countData)$countData)
colData(countData)
rowRanges(countData)

## -----------------------------------------------------------------------------
countData <- loadCountData(system.file("extdata", "sample_count_data.txt", package="CSSQ",mustWork = TRUE),system.file("extdata", "chr19_regions.bed", package="CSSQ"),sampleInfo)
countData
head(assays(countData)$countData)
colData(countData)
rowRanges(countData)

## -----------------------------------------------------------------------------
ansCountData <- ansTransform(countData)
ansCountData
head(assays(ansCountData)$ansCount)

## -----------------------------------------------------------------------------
normInfo <- normalizeData(ansCountData,numClusters=4)
normInfo
head(assays(normInfo)$normCount)

## -----------------------------------------------------------------------------
result <- DBAnalyze(normInfo,comparison=c("HSMM","HESC"))
result

## -----------------------------------------------------------------------------
#To let CSSQ calculate the counts
processedData <- preprocessData(system.file("extdata", "chr19_regions.bed", package="CSSQ"),system.file("extdata", "sample_info.txt", package="CSSQ"),sampleDir = system.file("extdata", package="CSSQ"),numClusters=4,noNeg=TRUE,plotData=FALSE)
#To provide CSSQ with the counts 
processedData <- preprocessData(system.file("extdata", "chr19_regions.bed", package="CSSQ"),system.file("extdata", "sample_info.txt", package="CSSQ"),inputCountData = system.file("extdata", "sample_count_data.txt", package="CSSQ",mustWork = TRUE),numClusters=4,noNeg=TRUE,plotData=FALSE)
#To find differential binding sites
result <- DBAnalyze(processedData,comparison=c("HSMM","HESC"))
processedData
result

## -----------------------------------------------------------------------------
sessionInfo()

#' Quantify region level count data
#'
#' The input is the set of regions and the sample information. It will 
#' calculate the number of reads falling in each region for each sample.
#' Returns a 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
#' object with regions, sample informationa and counts for all samples. 
#' 
#' @param regionBed A bed file containing the list of regions that are being 
#' analyzed.
#' @param sampleInfo Object from \code{\link{preprocessData}} containing 
#' sample information.
#' @param sampleDir Location of the input sample files in `sampleInfo` file.
#' (default: ".")
#' @param backgroundSubtract Logical indicating if background correction 
#' should be performed. (default: TRUE)
#' @param ... Additional arguments passed on to \code{\link{getBgSubVal}}.
#' 
#' @return \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
#' containing the regions, 
#'    sample information and counts for all samples.
#' @seealso \code{\link{getBgSubVal}} which this function calls. 
#' @import GenomicRanges
#' @import SummarizedExperiment
#' @import GenomicAlignments
#' @importFrom utils read.table
#' @import IRanges
#' @import Rsamtools
#' @import rtracklayer
#' @export
#' @examples
#' sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
#' package="CSSQ",mustWork = TRUE),sep="\t",header=TRUE)
#' countData <- getRegionCounts(system.file("extdata", "chr19_regions.bed", 
#' package="CSSQ"),sampleInfo,
#' sampleDir = system.file("extdata", package="CSSQ"))
#' countData
#' head(assays(countData)$countData)
#' colData(countData)
#' rowRanges(countData)

getRegionCounts <- function(regionBed,sampleInfo,sampleDir = ".",backgroundSubtract=TRUE,...){
    regionRange <- import(regionBed,format="BED")
    sampleInfo[,3] <- vapply(sampleInfo[,3], function(x) paste(sampleDir,x,sep="/"),character(1))
    sampleInfo[,5] <- vapply(sampleInfo[,5], function(x) paste(sampleDir,x,sep="/"),character(1))
    analysisInfo <- SummarizedExperiment(rowRanges=regionRange,colData=sampleInfo)
    if (ncol(sampleInfo) < 5){
        message ("No Input/Control sample information provided. Defaulting to no background correction.")
        NormbgSubCounts <- data.frame(vapply(seq_len(nrow(colData(analysisInfo))),function(x) getBgSubVal(analysisInfo,sampleIndex = x,backgroundSubtract=FALSE,...),double(length(regionRange))))
    }
    else{
        NormbgSubCounts <- data.frame(vapply(seq_len(nrow(colData(analysisInfo))),function(x) getBgSubVal(analysisInfo,sampleIndex = x,backgroundSubtract=backgroundSubtract,...),double(length(regionRange))))
    }
    colnames(NormbgSubCounts) <- colData(analysisInfo)[,1]
    countData <- SummarizedExperiment(assays = list(countData=NormbgSubCounts),rowRanges=regionRange,colData=sampleInfo)
    return(countData)
}

#' Plot data distribution histograms
#'
#' This function is to plot data distribution histogram before
#' and after anscombe transformation.
#' 
#' @param countData A 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' object from \code{\link{getRegionCounts}} with count data.
#' @param ansCount A 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' object from \code{\link{ansTransform}} with anscombe transformed data.
#' @param sampleName Name of the sample being plotted.
#' @param plotDataToPDF A logical parameter indicating whether to make plots of the
#' data distribution to a separate PDF file for each sample.
#' When TRUE, a histogram will be plotted for the data before and after 
#' transformation. 
#' When FALSE, no plots will be made. (default: FALSE)
#' 
#' @return A list of the histogram of the count data before and after
#' anscombe transformation if plotDataToPDF == FALSE. 
#' None if plotDataToPDF == TRUE.
#' @import ggplot2
#' @importFrom grDevices dev.off pdf
#' @export
#' @examples
#' exRange <- GRanges(seqnames=c("chr1","chr2","chr3","chr4"),
#' ranges=IRanges(start=c(1000,2000,3000,4000),end=c(1500,2500,3500,4500)))
#' sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
#' package="CSSQ",mustWork = TRUE),sep="\t",header=TRUE)
#' exCount <- matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),nrow=4,ncol=4)
#' exData <- SummarizedExperiment(assays = list(countData=exCount),
#' rowRanges=exRange,colData=sampleInfo)
#' ansExData <- ansTransform(exData)
#' plotEx <- plotDist(exData,ansExData,"HESC_R1")
#' plotEx[[1]]

plotDist <- function(countData,ansCount,sampleName,plotDataToPDF=FALSE) {
    sampeIndex <- try(which(as.character(colData(countData)[,1]) == sampleName)[[1]],silent=TRUE)
    if (inherits(sampeIndex, 'try-error')){stop("Invalid sample name")}
    countVec <- assays(countData)$countData[,sampeIndex]
    ansVec <- assays(ansCount)$ansCount[,sampeIndex]
    plot1 <- ggplot(data.frame(countVec), aes(countVec)) + geom_histogram(colour="black",fill="white",bins=100) + labs(title="Non transformed data",x="Signal level", y = "Frequency")+ theme_classic() + theme(plot.title = element_text(hjust = 0.5)) 
    plot2 <- ggplot(data.frame(ansVec), aes(ansVec)) + geom_histogram(colour="black",fill="white",bins=100) + labs(title="Anscombe transformed data",x="Anscombe transformed signal level", y = "Frequency")+ theme_classic() + theme(plot.title = element_text(hjust = 0.5))
    if (plotDataToPDF == FALSE){
    return(list(plot1,plot2))
    }
    else{
    pdf (paste(sampleName,"plots.pdf",sep="_"))
    print(plot1)
    print(plot2)
    dev.off()
    }
}

#' Identify possible combinations
#'
#' This function creates a data frame of all possbile combinations of sample 
#' labels.
#' This information is utilized by \code{\link{calculatePvalue}} to calculate 
#' the P-value using column permutation method.
#' 
#' @param trueLabel The true labels for the samples.
#' @param comparison A vector containing the comparison to be made. Names here 
#' need to correspond to the sample groups in the sample file (Eg. c("G1",G2") 
#' means the comparison G1/G2).
#' @param numSamples Number of samples in the dataset.
#' 
#' @return A data frame with possible combinations of samples other the true 
#' intended comparison.
#' @seealso \code{\link{DBAnalyze}} which calls this function and 
#' \code{\link{getNewLabels}} which this function calls
#' 
#' @importFrom utils combn

getComparisons <- function(trueLabel,comparison,numSamples){
    combns <- combn(seq_len(numSamples),numSamples/2)
    allLabels <- data.frame(vapply(seq_len(ncol(combns)/2),function(x) getNewLabels(trueLabel,comparison,numSamples,combns,x),character(numSamples)))
    returnLabels <- allLabels[,vapply(seq_len(ncol(allLabels)), function(x) !all(allLabels[,x] == trueLabel),logical(1))]
    return(returnLabels)
}

#' Perform quantification and normalization of count data
#'
#' This function quantifies each each region for a sample and performs
#' background correction and normalization as instructed. 
#' Returns a vector of count information for the input regions.
#' 
#' @param countData A 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' object from \code{\link{getRegionCounts}} with count data.
#' @param noNeg A Logical parameter indicating how to deal with negative 
#' values. 
#' When TRUE (default), all negative values will be moved to 0 before 
#' transforming. 
#' When FALSE, the signs will be maintained while the transformation will be 
#' applied to the absolute value. (default: TRUE) 
#' @param plotDataToPDF A logical parameter indicating whether to make plots of the
#' data distribution to a separate PDF file for each sample.
#' When TRUE, a histogram will be plotted for the data before and after 
#' transformation. 
#' When FALSE, no plots will be made. (default: FALSE)
#' 
#' @return A 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} object 
#' containing the anscombe transformed count data as the assay. 
#' @import GenomicRanges
#' @import SummarizedExperiment
#' @export
#' @examples
#' exRange <- GRanges(seqnames=c("chr1","chr2","chr3","chr4"),
#' ranges=IRanges(start=c(1000,2000,3000,4000),end=c(1500,2500,3500,4500)))
#' sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
#' package="CSSQ",mustWork = TRUE),sep="\t",header=TRUE)
#' exCount <- matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),nrow=4,ncol=4)
#' exData <- SummarizedExperiment(assays = list(countData=exCount),
#' rowRanges=exRange,colData=sampleInfo)
#' ansExData <- ansTransform(exData)
#' assays(ansExData)$ansCount

ansTransform <- function(countData,noNeg = TRUE,plotDataToPDF=FALSE) {
    rawData <- assays(countData)$countData
    if (noNeg == TRUE){
        rawData[rawData <0] <- 0
		newRowRanges <- rowRanges(countData)[apply(rawData, 1, function(x) !all(x==0)),]
		rawData <- rawData[apply(rawData, 1, function(x) !all(x==0)),]
        y <- 2*(sqrt(rawData+(3/8)))
		ansCount <- SummarizedExperiment(assays = list(ansCount=y),rowRanges=newRowRanges,colData=colData(countData))
    }
    else{
        y <- (2*(sqrt(abs(rawData)+(0))))*(abs(rawData)/rawData)
		ansCount <- SummarizedExperiment(assays = list(ansCount=y),rowRanges=rowRanges(countData),colData=colData(countData))
    }
    
    if (plotDataToPDF == TRUE){
    tmp <- vapply(seq_len(nrow(colData(countData))), function(x) plotDist(countData,ansCount,as.character(colData(countData)[,1][x]),plotDataToPDF=TRUE),integer(1))
    }
    return(ansCount)
}

#' Calculates P-value for the regions
#'
#' This calculates the adjusted P-values for the regions using column 
#' permutation and Benjamini Hochberg correction methods. 
#' 
#' @param trueTstat The T-statistics value calculated using 
#' \code{\link{calculateTvalue}} for the intended comparison.
#' @param compare_tstats The T-statistics value calculated using 
#' \code{\link{calculateTvalue}} for all other comparisons possible.
#' 
#' @return A vector that is the adjusted P-value for the intended comparison.
#' @seealso \code{\link{DBAnalyze}} which calls this function 
#' 
#' @importFrom stats p.adjust


calculatePvalue <- function(trueTstat,compare_tstats){
    abs_compare_tstats <- abs(compare_tstats)
    num_higher <- vapply(abs(trueTstat),function(x) (sum(abs_compare_tstats > x)),integer(1))
    numCompare <- ncol(compare_tstats)
    pval <- num_higher/(length(trueTstat) * numCompare)
    adjpval <- p.adjust(pval,method="BH",length(pval))
    return(adjpval)
}

#' Calculates Fold change values for the given label and comparison
#'
#' This calculates the Fold change for the given comparison.
#' 
#' @param preprocessedData A 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' object from \code{\link{preprocessData}}.
#' @param comparison A vector containing the comparison to be made. 
#' Names here need to correspond to the sample groups in the sample file 
#' (Eg. c("G1",G2") means the comparison G1/G2).
#' @param label A vector containing the labels to use for the samples in 
#' preprocessedData.
#' @param numSamples Number of samples in the dataset.
#' 
#' @return A vector that is the Fold change for the comparison and labels given.
#' @seealso \code{\link{DBAnalyze}} which calls this function 
#' 
#' @import GenomicRanges
#' @import SummarizedExperiment
#' @keywords internal

calculateFC <- function(preprocessedData,label,comparison,numSamples){
    norm_data <- assays(preprocessedData)$normCount
    colnames(norm_data) <- label
    restrucNormData <- cbind(norm_data[,names(norm_data) %in% (comparison[1])],norm_data[,names(norm_data) %in% (comparison[2])])
    FC <- data.frame(apply(restrucNormData,1,function(x) ifelse(mean(x[seq_len(numSamples/2)]) > mean(x[seq(((numSamples/2)+1),numSamples)]), mean(x[seq_len(numSamples/2)]) / mean(x[seq(((numSamples/2)+1),numSamples)]), -1*(mean(x[seq(((numSamples/2)+1),numSamples)])/mean(x[seq_len(numSamples/2)])))))
    colnames(FC)<- "FC"
    return(FC)
}

#' Perform k-means clustering, normalize anscombe data and calculate cluster 
#' variances for a sample. 
#'
#' This function performs normalization on the anscombe transformed data by
#' clustering them using k-means algorithmn and utilizing the information from
#' clusters. It returns an DataFrame object  normalized counts,
#' cluster information and the variance of that cluster for that sample. 
#' 
#' @param ansDataVec Anscombe transformed count data for a sample.
#' @param numClusters A number indicating the number of clusters to use for 
#' k-means clustering. (default: 4)
#'
#' @return DataFrame containing the normalized counts, 
#'    cluster information and the variance of the cluster in the sample. 
#' @seealso \code{\link{normalizeData}} which iterates over this function.
#' @import GenomicRanges
#' @import SummarizedExperiment
#' @importFrom S4Vectors DataFrame
#' @importFrom stats kmeans
#' @export
#' @examples
#' exCount <- c(1,2,3,4,5,6,7,8,9,10)
#' kmeansEx <- kmeansNormalize(exCount,numClusters=2)
#' kmeansEx

kmeansNormalize <- function(ansDataVec,numClusters=4) {
        y <- kmeans(ansDataVec,numClusters,nstart=20)
        b <- data.frame(y$centers)[order(data.frame(y$centers)$y.centers),,drop=FALSE]
        clusterLabels <- c(letters[seq_len(numClusters)])
        b$id <- clusterLabels
        b$id_o <- row.names(b)
        z <- data.frame(cbind(ansDataVec,y$cluster))
        colnames(z) <- c("vals","cluster")
        for(j in seq_len(numClusters)){
            z[z$cluster==b$id_o[j],][2] <- b$id[j]
        }
        cluster_data <- z$cluster
        calc_max <- (mean(z[z$cluster == clusterLabels[numClusters],]$vals)) + (3 * sd(z[z$cluster == clusterLabels[numClusters],]$vals))
        z$norm <- z$vals/calc_max
        varValues <- data.frame(vapply(seq_len(numClusters), function(x) var(z[z$cluster == clusterLabels[x],]$norm),double(1)))
        row.names(varValues) <- clusterLabels
        variance <- vapply(cluster_data,function(x) as.numeric(varValues[row.names(varValues)==x,][1]),double(1))
        names(variance) <- NULL
        norm_data <- z$norm
        return(list(clus=cluster_data,norm=norm_data,var=variance))
}

#' Labels the samples according to the combinations from \code{\link{getComparisons}}
#'
#' This function labels the samples according the combinations generated by 
#' \code{\link{getComparisons}}.
#' 
#' @param trueLabel The true labels for the samples.
#' @param comparison A vector containing the comparison to be made. Names here 
#' need to correspond to the sample groups in the sample file (Eg. c("G1",G2") 
#' means the comparison G1/G2).
#' @param numSamples Number of samples in the dataset.
#' @param combns Possible combinations of sample index generated in 
#' \code{\link{getComparisons}}.
#' @param index index of the combination to use for labeling.
#' 
#' @return A vector with labels.
#' @seealso \code{\link{getComparisons}} which calls this function
#' 

getNewLabels <- function(trueLabel,comparison,numSamples,combns,index){
    newLabel <- rep (comparison[1],numSamples)
    newLabel[combns[,index]] <- comparison[2]
    return(newLabel)
}

#' Calculates modified T-statistics values for the given label and comparison
#'
#' This calculates the modified T-statistics for the given comparison.
#' 
#' @param preprocessedData A 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' object from \code{\link{preprocessData}}.
#' @param comparison A vector containing the comparison to be made. Names here 
#' need to correspond to the sample groups in the sample file (Eg. c("G1",G2") 
#' means the comparison G1/G2).
#' @param label A vector containing the labels to use for the samples in 
#' preprocessedData.
#' @param numSamples Number of samples in the dataset.
#' 
#' @return A vector that is the modified T-statistics for the comparison and 
#' labels given.
#' @seealso \code{\link{DBAnalyze}} which calls this function 
#' 
#' @import GenomicRanges
#' @import SummarizedExperiment


calculateTvalue <- function(preprocessedData,label,comparison,numSamples){
    norm_data <- assays(preprocessedData)$normCount
    var_data <- assays(preprocessedData)$varData
    colnames(norm_data) <- label
    colnames(var_data) <- label
    restrucNormData <- cbind(norm_data[,names(norm_data) %in% (comparison[1])],norm_data[,names(norm_data) %in% (comparison[2])])
    restrucVarMatrix <- cbind(var_data[,names(var_data) %in% (comparison[1])],var_data[,names(var_data) %in% (comparison[2])])
    group1MeanVar <- rowMeans(restrucVarMatrix[,seq_len(numSamples/2)])
    group2MeanVar <- rowMeans(restrucVarMatrix[,((numSamples/2)+1):ncol(restrucVarMatrix)])
    denom <- sqrt(group1MeanVar/(numSamples/2) + group2MeanVar/(numSamples/2))
    combDT <- cbind(restrucNormData,denom)
    tstat <- apply(combDT,1,function(x) ((mean(x[seq_len(numSamples/2)])- mean(x[seq(((numSamples/2)+1),numSamples)]))/x[length(x)]))
    return(tstat)
}

#' Normalize anscombe transformed data
#'
#' This function iterates over \code{\link{kmeansNormalize}} to perform 
#' normalization for all samples in the dataset. It returns an 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
#' object  normalized counts, cluster information and the variance of that 
#' cluster for that sample. 
#' 
#' @param ansData 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} object 
#' from \code{\link{ansTransform}}.
#' @param numClusters A number indicating the number of clusters to use for 
#' k-means clustering. (default: 4)
#'
#' @return \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' containing the 
#' normalized counts, cluster information and the variance of the cluster in 
#' the sample. 
#' @seealso \code{\link{kmeansNormalize}} which this function calls. 
#' @import GenomicRanges
#' @import SummarizedExperiment
#' @export
#' @examples
#' exRange <- GRanges(seqnames=c("chr1","chr2","chr3","chr4"),
#' ranges=IRanges(start=c(1000,2000,3000,4000),end=c(1500,2500,3500,4500)))
#' sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
#' package="CSSQ",mustWork = TRUE),sep="\t",header=TRUE)
#' exCount <- matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),nrow=4,ncol=4)
#' exData <- SummarizedExperiment(assays = list(ansCount=exCount),
#' rowRanges=exRange,colData=sampleInfo)
#' normExData <- normalizeData(exData,numClusters=2)
#' assays(normExData)$normCount

normalizeData <- function(ansData,numClusters=4) {
    info <- vapply (seq_len(ncol(assays(ansData)$ansCount)), function(x) kmeansNormalize(assays(ansData)$ansCount[,x],numClusters=numClusters),list(character,double,double))
    clusterData <- data.frame(vapply(seq_len(ncol(info)),function(x) unlist(info[,x][1],use.names=FALSE),character(length(rowRanges(ansData)))))
    normCount <- data.frame(vapply(seq_len(ncol(info)),function(x) unlist(info[,x][2],use.names=FALSE),double(length(rowRanges(ansData)))))
    geneVars <- data.frame(vapply(seq_len(ncol(info)),function(x) unlist(info[,x][3],use.names=FALSE),double(length(rowRanges(ansData)))))
    colnames(clusterData) <- colData(ansData)[,1]
    colnames(normCount) <- colData(ansData)[,1]
    colnames(geneVars) <- colData(ansData)[,1]
    rownames(clusterData) <- rownames(ansData)
    rownames(normCount) <- rownames(ansData)
    rownames(geneVars) <- rownames(ansData)
    normInfo <- SummarizedExperiment(assays = list(normCount=normCount,clusterData=clusterData,varData=geneVars),rowRanges=rowRanges(ansData),colData=colData(ansData))
    return(normInfo)
}

#' Wrapper function to preprocess the data
#'
#' This is a wrapper function that calls the functions to preprocess the data.
#' It results in a 
#' \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} object
#'  normalized counts and meta data that can be used by  
#' \code{\link{DBAnalyze}}.
#'
#' @param inputRegions A bed file  the regions to analyze.
#' @param sampleInfoFile A tab separated file  all sample information. 
#' The following are the columns that are present in the file. 
#' * Sample Name : Names for the samples. 
#' * Group : The group the sample belongs.
#' * IP : The name of the sample bam file. 
#' * IP_aligned_reads : The number of aligned reads in the sample. This is used
#' in depth normalization process.
#' * IN : The name of the sample's control bam file.
#' * IN_aligned_reads : The number of aligned reads in the control file. This 
#' is used in depth normalization process.
#' @param sampleDir Location of the input sample files in `sampleInfoFile` file.
#' (default: ".")
#' Name,Group/Label,IP bam location,IP number of reads,IN bam location,
#' IN number of reads).
#' @param inputCountData The path to the file  count data. This parameter 
#' is used when directly loading count data from a file. This should be a tab 
#' separated file  sample names as header. 
#' @param numClusters A numerical parameter indicating the number of clusters 
#' to use in the normalization step. Passed on to \code{\link{normalizeData}}.
#' (default: 4)
#' @param noNeg A logical parameter indicating how to deal  negative 
#' values. It is passed to \code{\link{ansTransform}}. (default: TRUE)
#' @param plotDataToPDF A logical parameter indicating whether to make plots of the 
#' data distribution to a separate PDF file for each sample.
#' It is passed on to passed to \code{\link{ansTransform}}.
#' (default: FALSE)
#' @param ... Additional arguments passed on to \code{\link{getRegionCounts}}.
#' 
#' @return \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
#' containing the normalized counts, cluster information, the variance of the 
#' cluster in the sample and metadata. 
#' @seealso \code{\link{getRegionCounts}}, \code{\link{ansTransform}} and 
#' \code{\link{normalizeData}} which this function calls
#' 
#' @importFrom utils read.table
#' @export
#' @examples
#' processedData <- preprocessData(system.file("extdata", "chr19_regions.bed",
#' package="CSSQ"),system.file("extdata", "sample_info.txt", package="CSSQ"),
#' sampleDir = system.file("extdata", package="CSSQ"),
#' numClusters=4,noNeg=TRUE,plotDataToPDF=FALSE)
#' processedData

preprocessData <- function(inputRegions,sampleInfoFile,sampleDir = ".",inputCountData,numClusters=4,noNeg=TRUE,plotDataToPDF=FALSE,...){
    sampleInfo <- read.table(sampleInfoFile,header=TRUE,sep="\t")
    if (missing(inputCountData)){
    countData <- getRegionCounts(inputRegions,sampleInfo,sampleDir=sampleDir,...)
    }
    else{
    countData <- loadCountData(inputCountData,inputRegions,sampleInfo)
    }
    ansCountData <- ansTransform(countData,noNeg=noNeg,plotDataToPDF=plotDataToPDF)
    normInfo <- normalizeData(ansCountData,numClusters=numClusters)
    return (normInfo)
}

---
title: "ChIP-seq signal quantifier (CSSQ)"
shorttitle: "CSSQ"
package: CSSQ
author:
- name: Ashwath Kumar
  affiliation: Georgia Institute of Technology, Atlanta, GA
- name: Yajun Mei
  affiliation: Georgia Institute of Technology, Atlanta, GA
- name: Yuhong Fan
  affiliation: Georgia Institute of Technology, Atlanta, GA
  email: akumar301@gatech.edu
vignette: |
  %\VignetteIndexEntry{Introduction to CSSQ}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

```{r setup_knitr, include = FALSE, cache = FALSE}
library(BiocStyle)
# Decide whether to display parts for BioC (TRUE) or F1000 (FALSE)
on.bioc <- TRUE
library(knitr)
# Use fig.width = 7 for html and fig.width = 6 for pdf
fig.width <- ifelse(on.bioc, 8, 6)
knitr::opts_chunk$set(cache = 2, warning = FALSE, message = FALSE, error = FALSE,
  cache.path = "cache/", fig.path = "figure/", fig.width = fig.width)
```
# Introduction

This vignette introduces the package Bioconductor package `r Biocpkg("CSSQ")`. This package is designed to perform differential binding analysis for ChIP-seq samples. Differential binding is when there is a significant difference in the signal intensities at a region between two groups. The idea behind CSSQ is to implement a statistically robust pipeline which is built for ChIP-seq datasets to identify regions of differential binding among a set of interesting regions. It contains functions to quantify the signal over a predefined set of regions from the user, transform the data, normalize across samples and perform statistical tests to determine differential binding. There is an ever-increasing number of ChIP-seq samples, and this opens the door to compare these binding events between two groups of samples. CSSQ does this while considering the signal to noise ratios, paired control experiments and the lack of multiple (>2) replicates in a typical ChIP-seq experiment. It controls the false discovery rates while not compromising in its sensitivity. This document will give a brief overview of the processing steps.

# Processing summary

The workflow for `CSSQ` is as follows

1. Quantify regions of interest - A bed file with the regions of interest is used to count the number of reads aligning to these regions for the samples and their respective controls. 
2. Data processing - The raw count data is used to perform background correction, data transformation and normalization across samples. 
3. Identifying differential regions - The normalized count data is used to perform statistical tests to identify differentially bound regions. 

# Example

To use `CSSQ`, you will require a bed file containing the regions of interest as well as sample information file which contains the necessary information about the samples being analyzed. Below are examples of both files.  

```{r}
library(CSSQ)
regionBed <- read.table(system.file("extdata", "chr19_regions.bed", package="CSSQ",mustWork = TRUE))
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", package="CSSQ",mustWork = TRUE),sep="\t",header=TRUE)
head(regionBed)
sampleInfo
```

`CSSQ` provides two options for obtaining raw count data used to perform the analysis. The first option is to ask `CSSQ` to count the reads and perform background correction. The following commands will perform the steps and return an object with count data and meta data.

```{r}
countData <- getRegionCounts(system.file("extdata", "chr19_regions.bed", package="CSSQ"),sampleInfo,sampleDir = system.file("extdata", package="CSSQ"))
countData
head(assays(countData)$countData)
colData(countData)
rowRanges(countData)
```
The second option is to provide `CSSQ` with the count data in a tab separated format. The count data given will directly be used in transformation and normalization steps and no background correction will be performed. 

```{r}
countData <- loadCountData(system.file("extdata", "sample_count_data.txt", package="CSSQ",mustWork = TRUE),system.file("extdata", "chr19_regions.bed", package="CSSQ"),sampleInfo)
countData
head(assays(countData)$countData)
colData(countData)
rowRanges(countData)
```

Anscombe transformation is then applied to the count data using the `ansTransform` function. This function also has an option parameter to plot the data distribution of the data before and after transformation. 

```{r}
ansCountData <- ansTransform(countData)
ansCountData
head(assays(ansCountData)$ansCount)
```

Anscombe transformed data is then normalized across samples. Normalization in `CSSQ` using the `normalizeData` function. It takes as input the number of clusters to use in the underlying k-means algorithm that is used in the normalization process. It is advised to choose the number of clusters by analyzing the data distribution. 

```{r}
normInfo <- normalizeData(ansCountData,numClusters=4)
normInfo
head(assays(normInfo)$normCount)
```

The normalized data is then used by the `DBAnalyze` function to perform statistical tests to identify differentially bound regions. `CSSQ` utilized non-parametric approaches to calculate the test statistics and to calculate the p-value from the test statistics. This is done due to prevalent trend observed in ChIP-seq datasets of not having more than two replicates. The resulting `GRanges` object from this function contains the Benjamini Hochberg adjusted P-value and a Fold change for each of the input regions. 

```{r}
result <- DBAnalyze(normInfo,comparison=c("HSMM","HESC"))
result
```

For convenience, most of the above steps are wrapped in `preprocessData` function to perform them all at one go. All the above steps can be performed using the below commands. 

```{r}
#To let CSSQ calculate the counts
processedData <- preprocessData(system.file("extdata", "chr19_regions.bed", package="CSSQ"),system.file("extdata", "sample_info.txt", package="CSSQ"),sampleDir = system.file("extdata", package="CSSQ"),numClusters=4,noNeg=TRUE,plotData=FALSE)
#To provide CSSQ with the counts 
processedData <- preprocessData(system.file("extdata", "chr19_regions.bed", package="CSSQ"),system.file("extdata", "sample_info.txt", package="CSSQ"),inputCountData = system.file("extdata", "sample_count_data.txt", package="CSSQ",mustWork = TRUE),numClusters=4,noNeg=TRUE,plotData=FALSE)
#To find differential binding sites
result <- DBAnalyze(processedData,comparison=c("HSMM","HESC"))
processedData
result
```
# Session Info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculatePval.R
\name{calculatePvalue}
\alias{calculatePvalue}
\title{Calculates P-value for the regions}
\usage{
calculatePvalue(trueTstat, compare_tstats)
}
\arguments{
\item{trueTstat}{The T-statistics value calculated using 
\code{\link{calculateTvalue}} for the intended comparison.}

\item{compare_tstats}{The T-statistics value calculated using 
\code{\link{calculateTvalue}} for all other comparisons possible.}
}
\value{
A vector that is the adjusted P-value for the intended comparison.
}
\description{
This calculates the adjusted P-values for the regions using column 
permutation and Benjamini Hochberg correction methods.
}
\seealso{
\code{\link{DBAnalyze}} which calls this function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tval.R
\name{calculateTvalue}
\alias{calculateTvalue}
\title{Calculates modified T-statistics values for the given label and comparison}
\usage{
calculateTvalue(preprocessedData, label, comparison, numSamples)
}
\arguments{
\item{preprocessedData}{A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object from \code{\link{preprocessData}}.}

\item{label}{A vector containing the labels to use for the samples in 
preprocessedData.}

\item{comparison}{A vector containing the comparison to be made. Names here 
need to correspond to the sample groups in the sample file (Eg. c("G1",G2") 
means the comparison G1/G2).}

\item{numSamples}{Number of samples in the dataset.}
}
\value{
A vector that is the modified T-statistics for the comparison and 
labels given.
}
\description{
This calculates the modified T-statistics for the given comparison.
}
\seealso{
\code{\link{DBAnalyze}} which calls this function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getBgSubVal.R
\name{getBgSubVal}
\alias{getBgSubVal}
\title{Perform quantification and normalization of count data}
\usage{
getBgSubVal(
  analysisInfo,
  sampleIndex,
  normalizeReadDepth = TRUE,
  normalizeLength = FALSE,
  backgroundSubtract = TRUE,
  countMode = "Union",
  ignore.strand = TRUE,
  inter.feature = FALSE
)
}
\arguments{
\item{analysisInfo}{A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
object with regions and sample information from within 
\code{\link{getRegionCounts}}.}

\item{sampleIndex}{Index of the sample to process.}

\item{normalizeReadDepth}{Logical indicating if count data should be 
normalized for library sequencing depth. When TRUE (default), counts will 
be normalized for sequencing depth for each library. When FALSE, no such 
normalization will be performed and raw counts will be used. (default: TRUE)}

\item{normalizeLength}{Logical indicating if count data should be 
normalized to the length of the regions. When TRUE, count data will be 
normalized for the length of the region being analyzed. 
When FALSE (default), no such normalization will be performed. (default: FALSE)}

\item{backgroundSubtract}{Logical indicating if background correction 
should be performed.
When TRUE (default), background subtraction will be performed after 
length and depth normalization if applicable. When FALSE, no background 
subtraction will be performed. (default: TRUE)}

\item{countMode}{Count method passed on to 
summarizeOverlaps from GenomicAlignments package.
(\code{"Union"}, \code{"IntersectionNotEmpty"}, \code{"IntersectionStrict"} 
or \code{"user supplied function"}). (default: "Union")}

\item{ignore.strand}{A logical indicating if strand should be considered 
when matching. (default: TRUE)
Passed on to summarizeOverlaps from GenomicAlignments package.}

\item{inter.feature}{A logical indicating if the `r countMode` should be 
aware of overlapping features. When TRUE, reads mapping to multiple 
features are dropped (i.e., not counted). When FALSE (default), these reads
are retained and a count is assigned to each feature they map to. Passed on 
to summarizeOverlaps from GenomicAlignments package. (default: FALSE)}
}
\value{
A vector containing the counts for all the regions.
}
\description{
This function quantifies each each region for a sample and performs
background correction and normalization as instructed. 
Returns a vector of count information for the input regions.
}
\examples{
regionBed <- read.table(system.file("extdata", "chr19_regions.bed", 
package="CSSQ",mustWork = TRUE))
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
package="CSSQ",mustWork = TRUE),sep="\\t",header=TRUE)
sampleInfo[,3] <- sapply(sampleInfo[,3], 
function(x) system.file("extdata", x, package="CSSQ"))
sampleInfo[,5] <- sapply(sampleInfo[,5], 
function(x) system.file("extdata", x, package="CSSQ"))
regionRange <- GRanges(seqnames=regionBed$V1,
ranges=IRanges(start=regionBed$V2,end=regionBed$V3))
analysisInfo <- SummarizedExperiment(rowRanges=regionRange,
colData=sampleInfo)
NormbgSubCounts <- data.frame(sapply(c(1:nrow(colData(analysisInfo))),
function(x) getBgSubVal(analysisInfo,sampleIndex = x,backgroundSubtract=TRUE,
normalizeReadDepth=TRUE,normalizeLength=FALSE,countMode="Union", 
ignore.strand=TRUE,inter.feature=FALSE)))
NormbgSubCounts
}
\seealso{
\code{\link{getRegionCounts}} which calls this function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ansTransform.R
\name{ansTransform}
\alias{ansTransform}
\title{Perform quantification and normalization of count data}
\usage{
ansTransform(countData, noNeg = TRUE, plotDataToPDF = FALSE)
}
\arguments{
\item{countData}{A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object from \code{\link{getRegionCounts}} with count data.}

\item{noNeg}{A Logical parameter indicating how to deal with negative 
values. 
When TRUE (default), all negative values will be moved to 0 before 
transforming. 
When FALSE, the signs will be maintained while the transformation will be 
applied to the absolute value. (default: TRUE)}

\item{plotDataToPDF}{A logical parameter indicating whether to make plots of the
data distribution to a separate PDF file for each sample.
When TRUE, a histogram will be plotted for the data before and after 
transformation. 
When FALSE, no plots will be made. (default: FALSE)}
}
\value{
A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} object 
containing the anscombe transformed count data as the assay.
}
\description{
This function quantifies each each region for a sample and performs
background correction and normalization as instructed. 
Returns a vector of count information for the input regions.
}
\examples{
exRange <- GRanges(seqnames=c("chr1","chr2","chr3","chr4"),
ranges=IRanges(start=c(1000,2000,3000,4000),end=c(1500,2500,3500,4500)))
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
package="CSSQ",mustWork = TRUE),sep="\\t",header=TRUE)
exCount <- matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),nrow=4,ncol=4)
exData <- SummarizedExperiment(assays = list(countData=exCount),
rowRanges=exRange,colData=sampleInfo)
ansExData <- ansTransform(exData)
assays(ansExData)$ansCount
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotDist.R
\name{plotDist}
\alias{plotDist}
\title{Plot data distribution histograms}
\usage{
plotDist(countData, ansCount, sampleName, plotDataToPDF = FALSE)
}
\arguments{
\item{countData}{A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object from \code{\link{getRegionCounts}} with count data.}

\item{ansCount}{A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object from \code{\link{ansTransform}} with anscombe transformed data.}

\item{sampleName}{Name of the sample being plotted.}

\item{plotDataToPDF}{A logical parameter indicating whether to make plots of the
data distribution to a separate PDF file for each sample.
When TRUE, a histogram will be plotted for the data before and after 
transformation. 
When FALSE, no plots will be made. (default: FALSE)}
}
\value{
A list of the histogram of the count data before and after
anscombe transformation if plotDataToPDF == FALSE. 
None if plotDataToPDF == TRUE.
}
\description{
This function is to plot data distribution histogram before
and after anscombe transformation.
}
\examples{
exRange <- GRanges(seqnames=c("chr1","chr2","chr3","chr4"),
ranges=IRanges(start=c(1000,2000,3000,4000),end=c(1500,2500,3500,4500)))
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
package="CSSQ",mustWork = TRUE),sep="\\t",header=TRUE)
exCount <- matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),nrow=4,ncol=4)
exData <- SummarizedExperiment(assays = list(countData=exCount),
rowRanges=exRange,colData=sampleInfo)
ansExData <- ansTransform(exData)
plotEx <- plotDist(exData,ansExData,"HESC_R1")
plotEx[[1]]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRegionCounts.R
\name{getRegionCounts}
\alias{getRegionCounts}
\title{Quantify region level count data}
\usage{
getRegionCounts(
  regionBed,
  sampleInfo,
  sampleDir = ".",
  backgroundSubtract = TRUE,
  ...
)
}
\arguments{
\item{regionBed}{A bed file containing the list of regions that are being 
analyzed.}

\item{sampleInfo}{Object from \code{\link{preprocessData}} containing 
sample information.}

\item{sampleDir}{Location of the input sample files in `sampleInfo` file.
(default: ".")}

\item{backgroundSubtract}{Logical indicating if background correction 
should be performed. (default: TRUE)}

\item{...}{Additional arguments passed on to \code{\link{getBgSubVal}}.}
}
\value{
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
containing the regions, 
   sample information and counts for all samples.
}
\description{
The input is the set of regions and the sample information. It will 
calculate the number of reads falling in each region for each sample.
Returns a 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
object with regions, sample informationa and counts for all samples.
}
\examples{
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
package="CSSQ",mustWork = TRUE),sep="\\t",header=TRUE)
countData <- getRegionCounts(system.file("extdata", "chr19_regions.bed", 
package="CSSQ"),sampleInfo,
sampleDir = system.file("extdata", package="CSSQ"))
countData
head(assays(countData)$countData)
colData(countData)
rowRanges(countData)
}
\seealso{
\code{\link{getBgSubVal}} which this function calls.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmeansNormalize.R
\name{kmeansNormalize}
\alias{kmeansNormalize}
\title{Perform k-means clustering, normalize anscombe data and calculate cluster 
variances for a sample.}
\usage{
kmeansNormalize(ansDataVec, numClusters = 4)
}
\arguments{
\item{ansDataVec}{Anscombe transformed count data for a sample.}

\item{numClusters}{A number indicating the number of clusters to use for 
k-means clustering. (default: 4)}
}
\value{
DataFrame containing the normalized counts, 
   cluster information and the variance of the cluster in the sample.
}
\description{
This function performs normalization on the anscombe transformed data by
clustering them using k-means algorithmn and utilizing the information from
clusters. It returns an DataFrame object  normalized counts,
cluster information and the variance of that cluster for that sample.
}
\examples{
exCount <- c(1,2,3,4,5,6,7,8,9,10)
kmeansEx <- kmeansNormalize(exCount,numClusters=2)
kmeansEx
}
\seealso{
\code{\link{normalizeData}} which iterates over this function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getNewLabels.R
\name{getNewLabels}
\alias{getNewLabels}
\title{Labels the samples according to the combinations from \code{\link{getComparisons}}}
\usage{
getNewLabels(trueLabel, comparison, numSamples, combns, index)
}
\arguments{
\item{trueLabel}{The true labels for the samples.}

\item{comparison}{A vector containing the comparison to be made. Names here 
need to correspond to the sample groups in the sample file (Eg. c("G1",G2") 
means the comparison G1/G2).}

\item{numSamples}{Number of samples in the dataset.}

\item{combns}{Possible combinations of sample index generated in 
\code{\link{getComparisons}}.}

\item{index}{index of the combination to use for labeling.}
}
\value{
A vector with labels.
}
\description{
This function labels the samples according the combinations generated by 
\code{\link{getComparisons}}.
}
\seealso{
\code{\link{getComparisons}} which calls this function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getComparisons.R
\name{getComparisons}
\alias{getComparisons}
\title{Identify possible combinations}
\usage{
getComparisons(trueLabel, comparison, numSamples)
}
\arguments{
\item{trueLabel}{The true labels for the samples.}

\item{comparison}{A vector containing the comparison to be made. Names here 
need to correspond to the sample groups in the sample file (Eg. c("G1",G2") 
means the comparison G1/G2).}

\item{numSamples}{Number of samples in the dataset.}
}
\value{
A data frame with possible combinations of samples other the true 
intended comparison.
}
\description{
This function creates a data frame of all possbile combinations of sample 
labels.
This information is utilized by \code{\link{calculatePvalue}} to calculate 
the P-value using column permutation method.
}
\seealso{
\code{\link{DBAnalyze}} which calls this function and 
\code{\link{getNewLabels}} which this function calls
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizeData.R
\name{normalizeData}
\alias{normalizeData}
\title{Normalize anscombe transformed data}
\usage{
normalizeData(ansData, numClusters = 4)
}
\arguments{
\item{ansData}{\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} object 
from \code{\link{ansTransform}}.}

\item{numClusters}{A number indicating the number of clusters to use for 
k-means clustering. (default: 4)}
}
\value{
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
containing the 
normalized counts, cluster information and the variance of the cluster in 
the sample.
}
\description{
This function iterates over \code{\link{kmeansNormalize}} to perform 
normalization for all samples in the dataset. It returns an 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} 
object  normalized counts, cluster information and the variance of that 
cluster for that sample.
}
\examples{
exRange <- GRanges(seqnames=c("chr1","chr2","chr3","chr4"),
ranges=IRanges(start=c(1000,2000,3000,4000),end=c(1500,2500,3500,4500)))
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
package="CSSQ",mustWork = TRUE),sep="\\t",header=TRUE)
exCount <- matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),nrow=4,ncol=4)
exData <- SummarizedExperiment(assays = list(ansCount=exCount),
rowRanges=exRange,colData=sampleInfo)
normExData <- normalizeData(exData,numClusters=2)
assays(normExData)$normCount
}
\seealso{
\code{\link{kmeansNormalize}} which this function calls.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadCountData.R
\name{loadCountData}
\alias{loadCountData}
\title{Load count data from input file.}
\usage{
loadCountData(countFile, regionBed, sampleInfo)
}
\arguments{
\item{countFile}{A path to file containing the count data for the dataset. 
This should be a tab separated file  sample names as header.}

\item{regionBed}{A bed file containing the list of regions that are being 
analyzed.}

\item{sampleInfo}{Object from \code{\link{preprocessData}} containing sample
information.}
}
\value{
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object containing
the region information,
sample information and the count data.
}
\description{
It converts input count file and a bed file  regions 
into a  \code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object.
}
\examples{
countData <- loadCountData(system.file("extdata", "sample_count_data.txt", 
package="CSSQ",mustWork = TRUE),system.file("extdata", "chr19_regions.bed",
package="CSSQ"),
read.table(system.file("extdata", "sample_info.txt", package="CSSQ",
mustWork = TRUE),
sep="\\t",header=TRUE))
countData
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess.R
\name{preprocessData}
\alias{preprocessData}
\title{Wrapper function to preprocess the data}
\usage{
preprocessData(
  inputRegions,
  sampleInfoFile,
  sampleDir = ".",
  inputCountData,
  numClusters = 4,
  noNeg = TRUE,
  plotDataToPDF = FALSE,
  ...
)
}
\arguments{
\item{inputRegions}{A bed file  the regions to analyze.}

\item{sampleInfoFile}{A tab separated file  all sample information. 
The following are the columns that are present in the file. 
* Sample Name : Names for the samples. 
* Group : The group the sample belongs.
* IP : The name of the sample bam file. 
* IP_aligned_reads : The number of aligned reads in the sample. This is used
in depth normalization process.
* IN : The name of the sample's control bam file.
* IN_aligned_reads : The number of aligned reads in the control file. This 
is used in depth normalization process.}

\item{sampleDir}{Location of the input sample files in `sampleInfoFile` file.
(default: ".")
Name,Group/Label,IP bam location,IP number of reads,IN bam location,
IN number of reads).}

\item{inputCountData}{The path to the file  count data. This parameter 
is used when directly loading count data from a file. This should be a tab 
separated file  sample names as header.}

\item{numClusters}{A numerical parameter indicating the number of clusters 
to use in the normalization step. Passed on to \code{\link{normalizeData}}.
(default: 4)}

\item{noNeg}{A logical parameter indicating how to deal  negative 
values. It is passed to \code{\link{ansTransform}}. (default: TRUE)}

\item{plotDataToPDF}{A logical parameter indicating whether to make plots of the 
data distribution to a separate PDF file for each sample.
It is passed on to passed to \code{\link{ansTransform}}.
(default: FALSE)}

\item{...}{Additional arguments passed on to \code{\link{getRegionCounts}}.}
}
\value{
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
containing the normalized counts, cluster information, the variance of the 
cluster in the sample and metadata.
}
\description{
This is a wrapper function that calls the functions to preprocess the data.
It results in a 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}} object
 normalized counts and meta data that can be used by  
\code{\link{DBAnalyze}}.
}
\examples{
processedData <- preprocessData(system.file("extdata", "chr19_regions.bed",
package="CSSQ"),system.file("extdata", "sample_info.txt", package="CSSQ"),
sampleDir = system.file("extdata", package="CSSQ"),
numClusters=4,noNeg=TRUE,plotDataToPDF=FALSE)
processedData
}
\seealso{
\code{\link{getRegionCounts}}, \code{\link{ansTransform}} and 
\code{\link{normalizeData}} which this function calls
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FC.R
\name{calculateFC}
\alias{calculateFC}
\title{Calculates Fold change values for the given label and comparison}
\usage{
calculateFC(preprocessedData, label, comparison, numSamples)
}
\arguments{
\item{preprocessedData}{A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object from \code{\link{preprocessData}}.}

\item{label}{A vector containing the labels to use for the samples in 
preprocessedData.}

\item{comparison}{A vector containing the comparison to be made. 
Names here need to correspond to the sample groups in the sample file 
(Eg. c("G1",G2") means the comparison G1/G2).}

\item{numSamples}{Number of samples in the dataset.}
}
\value{
A vector that is the Fold change for the comparison and labels given.
}
\description{
This calculates the Fold change for the given comparison.
}
\seealso{
\code{\link{DBAnalyze}} which calls this function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbAnalyze.R
\name{DBAnalyze}
\alias{DBAnalyze}
\title{Performs differential binding analysis}
\usage{
DBAnalyze(preprocessedData, comparison)
}
\arguments{
\item{preprocessedData}{A 
\code{\link[SummarizedExperiment]{RangedSummarizedExperiment-class}}
object from \code{\link{preprocessData}}.}

\item{comparison}{A vector containing the comparison to be made. Names here 
need to correspond to the sample groups in the sample file (Eg. c("G1",G2") 
means the comparison G1/G2).}
}
\value{
A \code{\link[GenomicRanges]{GRanges-class}} object 
containing the regions along with their P-values and Fold change for the 
comparison.
}
\description{
This is a wrapper function that performs the different parts of
differential binding analysis. Returns a 
\code{\link[GenomicRanges]{GRanges-class}} with a calculated P-value and 
Fold change for each region.
}
\examples{
exRange <- GRanges(seqnames=c("chr1","chr2","chr3","chr4"),
ranges=IRanges(start=c(1000,2000,3000,4000),end=c(1500,2500,3500,4500)))
sampleInfo <- read.table(system.file("extdata", "sample_info.txt", 
package="CSSQ",mustWork = TRUE),sep="\\t",header=TRUE)
exCount <- matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),nrow=4,ncol=4)
exData <- SummarizedExperiment(assays = list(ansCount=exCount),
rowRanges=exRange,colData=sampleInfo)
normExData <- normalizeData(exData,numClusters=2)
res <- DBAnalyze(normExData,comparison=c("HSMM","HESC"))
res
}
